import json
import os
from argparse import ArgumentParser

import requests
import yaml
from yaml import YAMLError


def download_rules():
    parser = ArgumentParser(description="Publish DRE-C Dispatcher Rules to RMS")
    parser.add_argument("--env-name", required=True)
    parser.add_argument("--env-type", required=True)
    args = parser.parse_args()
    artifact_config = get_config_data("artifact")
    dre_c_rules_config = get_depl_manifest("dre-c-rules")
    major_version = get_dre_rule_config(
        dre_c_rules_config["environment"],
        args.env_type,
        args.env_name,
        "major_version",
    )
    art_url = f'{artifact_config["url"]}/{artifact_config["name"]}-{major_version}.json'
    response = requests.get(art_url)
    response.raise_for_status()

    path = os.path.join(
        os.path.dirname(__file__),
        f"../app/dre_c_rule_publisher/dre_c_rules/{args.env_type}/{args.env_name}",
    )

    if not os.path.exists(path):
        os.makedirs(path)

    with open(f"{path}/rules.json", "w") as outfile:
        json.dump(response.json(), outfile)


def get_depl_manifest(config_data_obj):
    config = os.path.join(
        os.path.dirname(__file__),
        "dre_c_rule_publisher/config/deployment-manifest.yml",
    )
    config_data = read_config_file(config)
    return config_data[config_data_obj]


def get_config_data(config_data_obj):
    config = os.path.join(
        os.path.dirname(__file__), "dre_c_rule_publisher/config/config.yml"
    )
    config_data = read_config_file(config)
    return config_data[config_data_obj]


def read_config_file(file):
    try:
        with open(file, "r") as fh:
            config_data = yaml.safe_load(os.path.expandvars(fh.read()))
            return config_data
    except YAMLError as yex:
        raise yex
    except FileNotFoundError as fex:
        raise fex
    except Exception as ex:
        raise ex


def get_dre_rule_config(environments, e_type, e_name, var):
    for env_type in environments:
        if env_type["type"]["name"] == e_type:
            for env in env_type["type"]["env"]:
                if env["name"] == e_name:
                    return env[var]

    raise Exception("Invalid Dre-C Config")


if __name__ == "__main__":
    download_rules()
